
# q-expansion of newform 1521.2.b.e, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1521
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[677, [1, 0]], [847, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [0, 0], [0, 1], [0, 0], [0, 0], [0, 0], [0, 8], [0, 0], [0, 0], [0, -7], [0, 10], [0, 0], [13, 0], [0, 0], [0, 0], [0, 0], [-13, 0], [0, 11], [0, 0], [0, -17], [-13, 0], [0, 0], [0, 0], [0, 5], [0, 0], [13, 0], [0, 0], [0, -19], [0, 0], [1, 0], [0, 0], [0, 0], [-7, 0], [0, 0], [0, 4], [11, 0], [0, 25], [0, 0], [0, 0], [0, 0], [-26, 0], [0, 0], [0, -23], [0, 0], [-17, 0], [-13, 0], [0, -28], [0, 0], [0, 22], [0, 0], [0, 0], [0, -14], [0, 0], [0, 0], [0, 0], [0, 0], [0, -29], [-26, 0], [0, 0], [25, 0], [0, 0], [0, -35], [0, 0], [-13, 0], [0, 0], [0, -31], [-29, 0], [0, 0], [0, -23], [0, 0], [0, 0], [-31, 0], [-13, 0], [0, -37], [0, 0], [0, 0], [0, 1], [0, 0], [0, -31], [0, 0], [0, -19], [0, 0], [-35, 0], [13, 0], [0, 0], [0, 0], [0, 41], [0, 0], [0, 43], [0, 0], [0, 0], [0, 44], [0, 0], [0, 32], [0, 0], [0, 0], [0, 0], [8, 0], [0, -17], [41, 0], [0, 0], [0, 0], [0, 0], [16, 0], [0, -46], [0, 0], [0, 0], [0, 0], [26, 0], [20, 0], [0, -37], [0, 0], [0, -17], [0, 1], [0, 0], [0, 47], [0, 0], [0, 0], [0, 0], [0, 49], [13, 0], [0, 0], [0, 0], [0, 41], [0, 0], [0, -53], [0, 0], [49, 0], [0, -7], [0, 16], [0, 0], [52, 0], [26, 0], [0, 0], [0, 2], [0, 0], [0, 25], [0, 0], [0, 0], [0, -19], [0, 0], [52, 0], [0, 0], [-53, 0], [0, 0], [0, -35], [0, 0], [-13, 0], [0, 0], [0, -34], [0, 0], [55, 0], [0, 0], [40, 0], [0, 0], [-52, 0], [0, 0], [26, 0], [0, 0], [0, 0], [0, 0], [0, 20], [0, 0], [0, 0], [0, 0], [44, 0], [59, 0], [0, 43], [0, 0], [0, 0], [0, -14], [0, 0], [0, 5], [52, 0], [0, 0], [0, -23], [0, 0], [-65, 0], [-13, 0], [0, 4], [0, 0], [65, 0], [0, 0], [0, 0], [0, 0], [-65, 0], [0, 32], [0, -29], [0, 0], [-7, 0], [0, 0], [11, 0], [0, 0], [0, 0], [0, 0], [0, 59], [-17, 0], [0, 0], [0, 0], [0, 0], [-19, 0], [0, 41], [-67, 0], [0, 0], [0, 0], [0, 71], [0, 0], [0, 0], [-56, 0], [25, 0], [0, 0], [-13, 0], [0, 0], [0, 0], [0, 22], [65, 0], [0, 0], [0, 0], [0, 0], [-31, 0], [0, 61], [0, 0], [0, -73], [0, 0], [13, 0], [0, 0], [0, 0], [-35, 0], [0, 0], [-65, 0], [-73, 0], [0, -76], [0, 0], [-37, 0], [0, 0], [0, -77], [0, 0], [0, 0], [0, 0], [0, 0], [64, 0], [-52, 0], [0, 11], [0, 0], [0, 0], [0, 79], [0, 0], [0, 47], [0, 0], [0, -29], [0, 0], [0, 0], [-77, 0], [0, 0], [0, 0], [26, 0], [0, 80], [0, 0], [0, -70], [-68, 0], [0, 0], [0, -7], [47, 0], [0, 0], [-13, 0], [0, 0], [0, 0], [0, 40], [0, 0], [49, 0], [0, -19], [0, 10], [52, 0], [65, 0], [0, -28], [0, 0], [0, 82], [0, 1], [0, 0], [0, 0], [0, -68], [0, 0], [0, -37], [0, -56], [0, 0], [65, 0], [0, 0], [0, -77], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [-85, 0], [0, 0], [-76, 0], [0, 0], [0, 0], [0, 49], [13, 0], [0, 0], [52, 0]]
